@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 module Doorkeeper
   module OpenidConnect
     module ResponseTypeConfig
@@ -12,6 +14,4 @@ module Doorkeeper
       end
     end
   end
-
-  Config.send :prepend, OpenidConnect::ResponseTypeConfig
 end
