@@ -1,10 +1,12 @@
+# frozen_string_literal: true
+
 require 'rails/generators/active_record'
 
 module Doorkeeper
   module OpenidConnect
     class MigrationGenerator < ::Rails::Generators::Base
       include ::Rails::Generators::Migration
-      source_root File.expand_path('../templates', __FILE__)
+      source_root File.expand_path('templates', __dir__)
       desc 'Installs Doorkeeper OpenID Connect migration file.'
 
       def install
