@@ -1,11 +1,13 @@
+# frozen_string_literal: true
+
 Doorkeeper::OpenidConnect.configure do
   issuer 'issuer string'
 
-  signing_key <<-EOL
------BEGIN RSA PRIVATE KEY-----
-....
------END RSA PRIVATE KEY-----
-EOL
+  signing_key <<~KEY
+    -----BEGIN RSA PRIVATE KEY-----
+    ....
+    -----END RSA PRIVATE KEY-----
+  KEY
 
   subject_types_supported [:public]
 
@@ -26,6 +28,18 @@ EOL
     # redirect_to new_user_session_url
   end
 
+  # Depending on your configuration, a DoubleRenderError could be raised
+  # if render/redirect_to is called at some point before this callback is executed.
+  # To avoid the DoubleRenderError, you could add these two lines at the beginning
+  #  of this callback: (Reference: https://github.com/rails/rails/issues/25106)
+  #   self.response_body = nil
+  #   @_response_body = nil
+  select_account_for_resource_owner do |resource_owner, return_to|
+    # Example implementation:
+    # store_location_for resource_owner, return_to
+    # redirect_to account_select_url
+  end
+
   subject do |resource_owner, application|
     # Example implementation:
     # resource_owner.id
