@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'rails_helper'
 
 describe Doorkeeper::OpenidConnect::DiscoveryController, type: :controller do
@@ -6,7 +8,7 @@ describe Doorkeeper::OpenidConnect::DiscoveryController, type: :controller do
       get :provider
       data = JSON.parse(response.body)
 
-      expect(data.sort).to eq({
+      expect(data.sort).to match({
         'issuer' => 'dummy',
         'authorization_endpoint' => 'http://test.host/oauth/authorize',
         'token_endpoint' => 'http://test.host/oauth/token',
@@ -16,14 +18,11 @@ describe Doorkeeper::OpenidConnect::DiscoveryController, type: :controller do
         'jwks_uri' => 'http://test.host/oauth/discovery/keys',
 
         'scopes_supported' => ['openid'],
+        'response_types_supported' => ['code', 'token', 'id_token', 'id_token token'],
+        'response_modes_supported' => %w[query fragment],
+        'grant_types_supported' => %w[authorization_code client_credentials implicit_oidc],
 
-        'response_types_supported' => ['code'],
-        'response_modes_supported' => ['query', 'fragment'],
-
-        'token_endpoint_auth_methods_supported' => [
-          'client_secret_basic',
-          'client_secret_post',
-        ],
+        'token_endpoint_auth_methods_supported' => %w[client_secret_basic client_secret_post],
 
         'subject_types_supported' => [
           'public',
@@ -37,24 +36,35 @@ describe Doorkeeper::OpenidConnect::DiscoveryController, type: :controller do
           'normal',
         ],
 
-        'claims_supported' => [
-          'iss',
-          'sub',
-          'aud',
-          'exp',
-          'iat',
-          'name',
-          'variable_name',
-          'created_at',
-          'updated_at',
-          'token_id',
-          'both_responses',
-          'id_token_response',
-          'user_info_response',
+        'claims_supported' => %w[
+          iss
+          sub
+          aud
+          exp
+          iat
+          name
+          variable_name
+          created_at
+          updated_at
+          token_id
+          both_responses
+          id_token_response
+          user_info_response
         ],
       }.sort)
     end
 
+    context 'when refresh_token grant type is enabled' do
+      before { Doorkeeper.configure { use_refresh_token } }
+
+      it 'add refresh_token to grant_types_supported' do
+        get :provider
+        data = JSON.parse(response.body)
+
+        expect(data['grant_types_supported']).to eq %w[authorization_code client_credentials refresh_token]
+      end
+    end
+
     it 'uses the protocol option for generating URLs' do
       Doorkeeper::OpenidConnect.configure do
         protocol { :testing }
@@ -65,6 +75,28 @@ describe Doorkeeper::OpenidConnect::DiscoveryController, type: :controller do
 
       expect(data['authorization_endpoint']).to eq 'testing://test.host/oauth/authorize'
     end
+
+    it 'does not return an end session endpoint if none is configured' do
+      get :provider
+      data = JSON.parse(response.body)
+
+      expect(data.key?('end_session_endpoint')).to be(false)
+    end
+
+    it 'uses the configured end session endpoint with self as context' do
+      Doorkeeper::OpenidConnect.configure do
+        end_session_endpoint -> { logout_url }
+      end
+
+      def controller.logout_url
+        'http://test.host/logout'
+      end
+
+      get :provider
+      data = JSON.parse(response.body)
+
+      expect(data['end_session_endpoint']).to eq 'http://test.host/logout'
+    end
   end
 
   describe '#webfinger' do
@@ -104,19 +136,19 @@ describe Doorkeeper::OpenidConnect::DiscoveryController, type: :controller do
     end
 
     context 'when using an RSA key' do
-      it_behaves_like 'a key response', expected_parameters: [:kty, :kid, :e, :n, :use, :alg]
+      it_behaves_like 'a key response', expected_parameters: %i[kty kid e n use alg]
     end
 
     context 'when using an EC key' do
       before { configure_ec }
 
-      it_behaves_like 'a key response', expected_parameters: [:kty, :kid, :crv, :x, :y, :use, :alg]
+      it_behaves_like 'a key response', expected_parameters: %i[kty kid crv x y use alg]
     end
 
     context 'when using an HMAC key' do
       before { configure_hmac }
 
-      it_behaves_like 'a key response', expected_parameters: [:kty, :kid, :use, :alg]
+      it_behaves_like 'a key response', expected_parameters: %i[kty kid use alg]
     end
   end
 end
