@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'rails_helper'
 
 describe Doorkeeper::AuthorizationsController, type: :controller do
@@ -16,8 +18,15 @@ describe Doorkeeper::AuthorizationsController, type: :controller do
     }.merge(params)
   end
 
-  def build_redirect_uri(params = {})
-    Doorkeeper::OAuth::Authorization::URIBuilder.uri_with_query(application.redirect_uri, params)
+  def build_redirect_uri(params = {}, type: 'query')
+    case type
+    when 'query'
+      Doorkeeper::OAuth::Authorization::URIBuilder.uri_with_query(application.redirect_uri, params)
+    when 'fragment'
+      Doorkeeper::OAuth::Authorization::URIBuilder.uri_with_fragment(application.redirect_uri, params)
+    else
+      raise ArgumentError, "Unsupported uri type #{type}"
+    end
   end
 
   def expect_authorization_form!
@@ -31,6 +40,12 @@ describe Doorkeeper::AuthorizationsController, type: :controller do
   end
 
   describe '#authenticate_resource_owner!' do
+    it 'redirects to login form when not logged in' do
+      authorize! current_user: nil
+
+      expect(response).to redirect_to '/login'
+    end
+
     context 'with OIDC requests' do
       before do
         expect(controller).to receive(:handle_oidc_prompt_param!)
@@ -42,12 +57,6 @@ describe Doorkeeper::AuthorizationsController, type: :controller do
 
         expect_authorization_form!
       end
-
-      it 'redirects to login form when not logged in' do
-        authorize! current_user: nil
-
-        expect(response).to redirect_to '/login'
-      end
     end
 
     context 'with non-OIDC requests' do
@@ -57,9 +66,31 @@ describe Doorkeeper::AuthorizationsController, type: :controller do
       end
 
       it 'when action is not :new' do
-        get :show
+        get :show, params: {
+          response_type: 'code',
+          current_user: user.id,
+          client_id: application.uid,
+          scope: default_scopes,
+          redirect_uri: application.redirect_uri,
+        }
 
-        expect(response).to redirect_to '/login'
+        expect(response).to render_template('doorkeeper/authorizations/show')
+      end
+
+      context 'when pre_authorization is invalid' do
+        it 'render error when client_id is missing' do
+          authorize!(client_id: nil)
+
+          expect(response).to be_successful
+          expect(response).to render_template('doorkeeper/authorizations/error')
+        end
+
+        it 'render error when response_type is missing' do
+          authorize!(response_type: nil)
+
+          expect(response).to be_successful
+          expect(response).to render_template('doorkeeper/authorizations/error')
+        end
       end
 
       it 'when openid scope is not present' do
@@ -67,13 +98,6 @@ describe Doorkeeper::AuthorizationsController, type: :controller do
 
         expect_authorization_form!
       end
-
-      it 'when client_id and scope are missing' do
-        authorize!(client_id: nil, scope: nil)
-
-        expect(response).to be_successful
-        expect(response).to render_template('doorkeeper/authorizations/error')
-      end
     end
   end
 
@@ -101,19 +125,33 @@ describe Doorkeeper::AuthorizationsController, type: :controller do
 
           error_params = {
             'error' => 'invalid_request',
-            'error_description' => 'The request is missing a required parameter, includes an unsupported parameter value, or is otherwise malformed.'
+            'error_description' => 'The request is missing a required parameter, includes an unsupported parameter value, or is otherwise malformed.',
           }
 
           expect(response.status).to redirect_to build_redirect_uri(error_params)
           expect(JSON.parse(response.body)).to eq(error_params)
         end
 
+        it 'uses the fragment style uris when redirecting an invalid request error for implicit flow request' do
+          allow(Doorkeeper.configuration).to receive(:grant_flows).and_return(['implicit_oidc'])
+
+          authorize! response_type: 'id_token token', prompt: 'none login'
+
+          error_params = {
+            'error' => 'invalid_request',
+            'error_description' => 'The request is missing a required parameter, includes an unsupported parameter value, or is otherwise malformed.',
+          }
+
+          expect(response.status).to redirect_to build_redirect_uri(error_params, type: 'fragment')
+          expect(JSON.parse(response.body)).to eq(error_params)
+        end
+
         context 'when not logged in' do
           let(:error_params) do
             {
               'error' => 'login_required',
               'error_description' => 'The authorization server requires end-user authentication',
-              'state' => 'somestate'
+              'state' => 'somestate',
             }
           end
 
@@ -124,33 +162,61 @@ describe Doorkeeper::AuthorizationsController, type: :controller do
             expect(JSON.parse(response.body)).to eq(error_params)
           end
 
-          it 'does not redirect to an invalid redirect_uri' do
-            authorize! prompt: 'none', current_user: nil, state: 'somestate', redirect_uri: 'https://evilapp.com'
+          it 'uses the fragment style uris when redirecting an error for implicit flow request' do
+            allow(Doorkeeper.configuration).to receive(:grant_flows).and_return(['implicit_oidc'])
 
-            expect(response).not_to be_redirect
-            expect(response.status).to eq(Doorkeeper.gem_version >= Gem::Version.new('5.1') ? 400 : 401)
-            expect(JSON.parse(response.body)).to eq error_params.merge(
-              'error' => 'invalid_redirect_uri',
-              'error_description' => "The requested redirect uri is malformed or doesn't match client redirect URI."
-            )
+            authorize! response_type: 'id_token token', prompt: 'none', current_user: nil, state: 'somestate'
+
+            expect(response).to redirect_to build_redirect_uri(error_params, type: 'fragment')
+            expect(JSON.parse(response.body)).to eq(error_params)
           end
         end
       end
 
       context 'and no matching token' do
         it 'redirects to the callback if skip_authorization is set to true' do
-          allow(controller).to receive(:skip_authorization?) { true }
+          allow(controller).to receive(:skip_authorization?).and_return(true)
 
           authorize! prompt: 'none'
           expect_successful_callback!
         end
 
+        context 'when not logged in' do
+          it 'returns the login_required error when not logged in' do
+            error_params = {
+              'error' => 'login_required',
+              'error_description' => 'The authorization server requires end-user authentication',
+              'state' => 'somestate',
+            }
+
+            authorize! prompt: 'none', current_user: nil, state: 'somestate'
+
+            expect(response).to redirect_to build_redirect_uri(error_params)
+            expect(JSON.parse(response.body)).to eq(error_params)
+          end
+
+          it 'uses the fragment style uris when redirecting an error for implicit flow request' do
+            allow(Doorkeeper.configuration).to receive(:grant_flows).and_return(['implicit_oidc'])
+
+            error_params = {
+              'error' => 'login_required',
+              'error_description' => 'The authorization server requires end-user authentication',
+              'state' => 'somestate',
+            }
+
+            authorize! response_type: 'id_token token', prompt: 'none', current_user: nil, state: 'somestate'
+
+            expect(response).to redirect_to build_redirect_uri(error_params, type: 'fragment')
+            expect(JSON.parse(response.body)).to eq(error_params)
+          end
+        end
+
         it 'returns a consent_required error when logged in' do
           authorize! prompt: 'none'
 
           error_params = {
             'error' => 'consent_required',
-            'error_description' => 'The authorization server requires end-user consent'
+            'error_description' => 'The authorization server requires end-user consent',
           }
 
           expect(response).to redirect_to build_redirect_uri(error_params)
@@ -183,38 +249,38 @@ describe Doorkeeper::AuthorizationsController, type: :controller do
     end
 
     context 'with a prompt=select_account parameter' do
-      it 'returns an account_selection_required error' do
+      it 'redirects to the select account screen' do
         authorize! prompt: 'select_account'
 
-        error_params = {
-          'error' => 'account_selection_required',
-          'error_description' => 'The authorization server requires end-user account selection'
-        }
-
-        expect(response.status).to redirect_to build_redirect_uri(error_params)
-        expect(JSON.parse(response.body)).to eq(error_params)
+        expect(response).to redirect_to('/select_account')
       end
     end
 
     context 'with an unknown prompt parameter' do
-        it 'returns an invalid_request error' do
-          authorize! prompt: 'maybe'
+      it 'returns an invalid_request error' do
+        authorize! prompt: 'maybe'
 
-          error_params = {
-            'error' => 'invalid_request',
-            'error_description' => 'The request is missing a required parameter, includes an unsupported parameter value, or is otherwise malformed.'
-          }
+        error_params = {
+          'error' => 'invalid_request',
+          'error_description' => 'The request is missing a required parameter, includes an unsupported parameter value, or is otherwise malformed.',
+        }
 
-          expect(response.status).to redirect_to build_redirect_uri(error_params)
-          expect(JSON.parse(response.body)).to eq(error_params)
-        end
+        expect(response).to redirect_to build_redirect_uri(error_params)
+        expect(JSON.parse(response.body)).to eq(error_params)
+      end
+
+      it 'does not redirect to an invalid redirect_uri' do
+        authorize! prompt: 'maybe', redirect_uri: 'https://evilapp.com'
+
+        expect(response).not_to be_redirect
+      end
     end
   end
 
   describe '#handle_oidc_max_age_param!' do
     context 'with an invalid max_age parameter' do
       it 'renders the authorization form' do
-        %w[ 0 -1 -23 foobar ].each do |max_age|
+        %w[0 -1 -23 foobar].each do |max_age|
           authorize! max_age: max_age
 
           expect_authorization_form!
@@ -251,7 +317,7 @@ describe Doorkeeper::AuthorizationsController, type: :controller do
 
     before do
       allow(subject).to receive(:performed?) { performed }
-      allow(subject.request).to receive(:path) { '/oauth/authorize' }
+      allow(subject.request).to receive(:path).and_return('/oauth/authorize')
       allow(subject.request).to receive(:query_parameters) {
         { client_id: 'foo', prompt: 'login consent select_account' }.with_indifferent_access
       }
@@ -294,4 +360,47 @@ describe Doorkeeper::AuthorizationsController, type: :controller do
       end
     end
   end
+
+  describe '#select_account_for_resource_owner' do
+    before do
+      allow(subject.request).to receive(:path).and_return('/oauth/authorize')
+      allow(subject.request).to receive(:query_parameters) {
+        { client_id: 'foo', prompt: 'login consent select_account' }.with_indifferent_access
+      }
+    end
+
+    def select_account!
+      passed_args = nil
+
+      Doorkeeper::OpenidConnect.configure do
+        select_account_for_resource_owner do |*args|
+          passed_args = args
+        end
+      end
+
+      subject.send :select_account_for_oidc_resource_owner, user
+      passed_args
+    end
+
+    it 'calls select_account_for_resource_owner with the current user and the return path' do
+      resource_owner, return_to = select_account!
+
+      expect(resource_owner).to eq user
+      expect(return_to).to eq '/oauth/authorize?client_id=foo&prompt=login+consent'
+    end
+
+    it 'removes select_account from the prompt parameter and keeps other values' do
+      _, return_to = select_account!
+      return_params = Rack::Utils.parse_query(URI.parse(return_to).query)
+
+      expect(return_params['prompt']).to eq 'login consent'
+    end
+  end
+
+  describe '#pre_auth' do
+    it 'permits nonce parameter' do
+      authorize! nonce: '123456'
+      expect(assigns(:pre_auth).nonce).to eq '123456'
+    end
+  end
 end
