@@ -1,11 +1,13 @@
+# frozen_string_literal: true
+
 require 'rails_helper'
 
 describe Doorkeeper::OAuth::IdTokenTokenResponse do
-  subject { Doorkeeper::OAuth::IdTokenTokenResponse.new(pre_auth, auth, id_token) }
+  subject { described_class.new(pre_auth, auth, id_token) }
+
   let(:token) { create :access_token }
   let(:application) do
-    scopes = double(all: ['public'])
-    double(:application, id: 9990, scopes: scopes)
+    create(:application, scopes: 'public')
   end
   let(:pre_auth) do
     double(
@@ -19,9 +21,14 @@ describe Doorkeeper::OAuth::IdTokenTokenResponse do
       nonce: '12345'
     )
   end
+  let(:owner) { build_stubbed(:user) }
   let(:auth) do
-    Doorkeeper::OAuth::Authorization::Token.new(pre_auth, double(id: 1)).tap do |c|
-      c.issue_token
+    Doorkeeper::OAuth::Authorization::Token.new(pre_auth, owner).tap do |c|
+      if c.respond_to?(:issue_token!)
+        c.issue_token!
+      else
+        c.issue_token
+      end
     end
   end
   let(:id_token) { Doorkeeper::OpenidConnect::IdToken.new(token, pre_auth) }
